@@ -5,7 +5,9 @@ import base64
 import hashlib
 import logging
 import json
+import math
 from pathlib import Path
+from os.path import exists
 from Crypto import Random
 from jinja2 import Template
 from Crypto.Cipher import AES
@@ -21,12 +23,12 @@ except ImportError:
     string_types = str
 
 JS_LIBRARIES = [
-    '//cdnjs.cloudflare.com/ajax/libs/crypto-js/4.1.1/core.js',
-    '//cdnjs.cloudflare.com/ajax/libs/crypto-js/4.1.1/enc-base64.js',
-    '//cdnjs.cloudflare.com/ajax/libs/crypto-js/4.1.1/cipher-core.js',
-    '//cdnjs.cloudflare.com/ajax/libs/crypto-js/4.1.1/pad-nopadding.js',
-    '//cdnjs.cloudflare.com/ajax/libs/crypto-js/4.1.1/md5.js',
-    '//cdnjs.cloudflare.com/ajax/libs/crypto-js/4.1.1/aes.js'
+    ['//cdnjs.cloudflare.com/ajax/libs/crypto-js/4.1.1/core.js','b55ae8027253d4d54c4f1ef3b6254646'],
+    ['//cdnjs.cloudflare.com/ajax/libs/crypto-js/4.1.1/enc-base64.js','f551ce1340a86e5edbfef4a6aefa798f'],
+    ['//cdnjs.cloudflare.com/ajax/libs/crypto-js/4.1.1/cipher-core.js','dfddc0e33faf7a794e0c3c140544490e'],
+    ['//cdnjs.cloudflare.com/ajax/libs/crypto-js/4.1.1/pad-nopadding.js','e288e14e2cd299c3247120114e1178e6'],
+    ['//cdnjs.cloudflare.com/ajax/libs/crypto-js/4.1.1/md5.js','349498f298a6e6e6a85789d637e89109'],
+    ['//cdnjs.cloudflare.com/ajax/libs/crypto-js/4.1.1/aes.js','da81b91b1b57c279c29b3469649d9b86'],
 ]
 
 PLUGIN_DIR = os.path.dirname(os.path.realpath(__file__))
@@ -62,7 +64,6 @@ class encryptContentPlugin(BasePlugin):
         ('global_password', config_options.Type(string_types, default=None)),
         ('use_secret', config_options.Type(string_types, default=None)),
         ('ignore_missing_secret', config_options.Type(bool, default=False)),
-        ('password', config_options.Type(string_types, default=None)),
         ('remember_password', config_options.Type(bool, default=False)),
         ('session_storage', config_options.Type(bool, default=True)),
         ('default_expire_delay', config_options.Type(int, default=int(24))),
@@ -83,19 +84,42 @@ class encryptContentPlugin(BasePlugin):
         ('inject', config_options.Type(dict, default={})),
         ('selfhost', config_options.Type(bool, default=False)),
         ('selfhost_download', config_options.Type(bool, default=True)),
+        ('selfhost_dir', config_options.Type(string_types, default='')),
         ('translations', config_options.Type(dict, default={}, required=False)),
+        ('hash_filenames', config_options.Type(dict, default={}, required=False)),
         # legacy features, doesn't exist anymore
-        ('disable_cookie_protection', config_options.Type(bool, default=False)),
-        ('decrypt_search', config_options.Type(bool, default=False)),
-        ('experimental', config_options.Type(bool, default=False)),
     )
 
+    setup = {}
+
     def __hash_md5__(self, text):
         """ Creates an md5 hash from text. """
         key = hashlib.md5()
         key.update(text.encode('utf-8'))
         return key.digest()
 
+    def __hash_md5_file__(self, fname):
+        hash_md5 = hashlib.md5()
+        with open(fname, "rb") as f:
+            for chunk in iter(lambda: f.read(4096), b""):
+                hash_md5.update(chunk)
+        return hash_md5.hexdigest()
+
+    def __download_and_check__(self, filename, url, hash):
+        hash_md5 = hashlib.md5()
+        if not exists(filename):
+            with urlopen(url) as response:
+                filecontent = response.read()
+                hash_md5.update(filecontent)
+                hash_check = hash_md5.hexdigest()
+                if hash == hash_check:
+                    with open(filename, 'wb') as file:
+                        file.write(filecontent)
+                        logger.info('Downloaded external asset "' + filename.name + '"')
+                else:
+                    logger.error('Error downloading asset "' + filename.name + '" hash mismatch!')
+                    os._exit(1)
+
     def __encrypt_text_aes__(self, text, password):
         """ Encrypts text with AES-256. """
         BLOCK_SIZE = 32
@@ -113,28 +137,36 @@ class encryptContentPlugin(BasePlugin):
             PADDING_CHAR
         )
 
-    def __encrypt_content__(self, content, password, base_path, encryptcontent_path):
+    def __encrypt_content__(self, content, base_path, encryptcontent_path, encryptcontent):
         """ Replaces page or article content with decrypt form. """
-        ciphertext_bundle = self.__encrypt_text_aes__(content, password)
+        ciphertext_bundle = self.__encrypt_text_aes__(content, encryptcontent['password'])
 
         # optionally selfhost cryptojs
-        if self.config["selfhost"]:
-            js_libraries = []
-            for jsurl in JS_LIBRARIES:
-                js_libraries.append(base_path + 'assets/javascripts/cryptojs/' + jsurl.rsplit('/',1)[1])
-        else:
-            js_libraries = JS_LIBRARIES
+        js_libraries = []
+        for jsurl in JS_LIBRARIES:
+            if self.config["selfhost"]:
+                js_libraries.append(base_path + 'assets/javascripts/cryptojs/' + jsurl[0].rsplit('/',1)[1])
+            else:
+                js_libraries = jsurl[0]
 
-        decrypt_form = Template(self.config['html_template']).render({
+        obfuscate = encryptcontent.get('obfuscate')
+        if obfuscate:
+            obfuscate_password = encryptcontent['password']
+        else:
+            obfuscate_password = None
+
+        decrypt_form = Template(self.setup['html_template']).render({
             # custom message and template rendering
-            'summary': self.config['summary'],
-            'placeholder': self.config['placeholder'],
+            'summary': encryptcontent['summary'],
+            'placeholder': encryptcontent['placeholder'],
             'password_button': self.config['password_button'],
-            'password_button_text': self.config['password_button_text'],
-            'encryption_info_message': self.config['encryption_info_message'],
-            'decryption_failure_message': json.dumps(self.config['decryption_failure_message']),
+            'password_button_text': encryptcontent['password_button_text'],
+            'encryption_info_message': encryptcontent['encryption_info_message'],
+            'decryption_failure_message': json.dumps(encryptcontent['decryption_failure_message']),
             'input_class': self.config['input_class'],
             'button_class': self.config['button_class'],
+            'obfuscate': obfuscate,
+            'obfuscate_password': obfuscate_password,
             # this benign decoding is necessary before writing to the template,
             # otherwise the output string will be wrapped with b''
             'ciphertext_bundle': b';'.join(ciphertext_bundle).decode('ascii'),
@@ -148,7 +180,7 @@ class encryptContentPlugin(BasePlugin):
 
     def __generate_decrypt_js__(self):
         """ Generate JS file with enable feature. """
-        decrypt_js = Template(self.config['js_template']).render({
+        decrypt_js = Template(self.setup['js_template']).render({
             # custom message and template rendering
             'password_button': self.config['password_button'],
             # enable / disable features
@@ -158,15 +190,54 @@ class encryptContentPlugin(BasePlugin):
             'remember_password': self.config['remember_password'],
             'session_storage': self.config['session_storage'],
             'default_expire_delay': int(self.config['default_expire_delay']),
-            'encrypted_something': self.config['encrypted_something'],
+            'encrypted_something': self.setup['encrypted_something'],
             'reload_scripts': self.config['reload_scripts'],
             'experimental': self.config['search_index'] == 'dynamically',
-            'site_path': self.config['site_path'],
+            'site_path': self.setup['site_path'],
             # add extra vars
             'extra': self.config['js_extra_vars']
         })
         return decrypt_js
 
+    def __get_entropy_from_password__(self, password):
+        #          123456789012345678901234567890
+        lcase =   "abcdefghijklmnopqrstuvwxyz"
+        ucase =   "ABCDEFGHIJKLMNOPQRTSUVWXYZ"
+        number =  "0123456789"
+        special = "!*#,;?+-_.=~^%()[]{}|:/"
+        other = ""
+        lcase_used = False
+        ucase_used = False
+        number_used = False
+        special_used = False
+        other_used = 0
+        for character in password:
+            if character in lcase:
+                lcase_used = True
+            elif character in ucase:
+                ucase_used = True
+            elif character in number:
+                number_used = True
+            elif character in special:
+                special_used = True
+            elif character not in other:
+                other_used += 1
+                other += character
+        ent = 0
+        if lcase_used:
+            ent += len(lcase)
+        if ucase_used:
+            ent += len(ucase)
+        if number_used:
+            ent += len(number)
+        if special_used:
+            ent += len(special)
+        ent += other_used
+        ent_max = len(lcase) + len(ucase) + len(number) + len(special) + len(other)
+        enttropy_spied_on = math.log( pow(ent, len(password)) ) / math.log(2)
+        enttropy_secret = math.log( pow(ent_max, len(password)) ) / math.log(2)
+        return enttropy_spied_on, enttropy_secret
+
     # MKDOCS Events builds
 
     def on_config(self, config, **kwargs):
@@ -181,10 +252,11 @@ class encryptContentPlugin(BasePlugin):
         # Override default template
         logger.debug('Load HTML template from file: "{file}".'.format(file=str(self.config['html_template_path'])))
         with open(self.config['html_template_path'], 'r') as template_html:
-            self.config['html_template'] = template_html.read()
+            self.setup['html_template'] = template_html.read()
         logger.debug('Load JS template from file: "{file}".'.format(file=str(self.config['js_template_path'])))
         with open(self.config['js_template_path'], 'r') as template_js:
-            self.config['js_template'] = template_js.read()
+            self.setup['js_template'] = template_js.read()
+
         # Optionnaly use Github secret
         if self.config.get('use_secret'):
             if os.environ.get(str(self.config['use_secret'])):
@@ -226,30 +298,30 @@ class encryptContentPlugin(BasePlugin):
             self.config['mermaid2'] = False
         # Warn about deprecated features on Vervion 2.0.0
         deprecated_options_detected = False
-        if self.config.get('disable_cookie_protection'):
-            logger.warning('DEPRECATED: Feature "disable_cookie_protection" is no longer supported. Can by remove.')
-            deprecated_options_detected = True
-        if self.config.get('decrypt_search'):
-            logger.warning('DEPRECATED: Feature "decrypt_search" is no longer supported. Use search_index on "clear" mode instead.')
-            deprecated_options_detected = True
-            logger.info('Fallback "decrypt_search" configuraiton to "search_index" mode clear.')
-            self.config['search_index'] = 'clear'
+
         if deprecated_options_detected:
             logger.warning('DEPRECATED: Features marked as deprecated will be remove in next minor version !')
-        # Re order plugins to be sure search-index are not encrypted
-        if self.config['search_index'] == 'clear':
-            logger.debug('Reordering plugins loading and put search and encryptcontent at the end of the event pipe.')
-            try: #PluginCollection is no instance of OrderedDict anymore since MkDocs 1.4
-                config['plugins'].move_to_end('search')
-                config['plugins'].move_to_end('encryptcontent')
-            except:
-                logger.warning('Could not reorder plugins. Please check that "search" and "encryptcontent" are at the end of plugins')
         # Enable experimental code .. :popcorn:
         elif self.config['search_index'] == 'dynamically':
             logger.info("EXPERIMENTAL search index encryption enabled.")
-        self.config['encrypted_something'] = {**self.config['inject'], **self.config['encrypted_something']} #add inject to encrypted_something
+        self.setup['encrypted_something'] = {**self.config['inject'], **self.config['encrypted_something']} #add inject to encrypted_something
         # Get path to site in case of subdir in site_url
-        self.config['site_path'] = urlsplit(config.data["site_url"] or '/').path[1::]
+        self.setup['site_path'] = urlsplit(config.data["site_url"] or '/').path[1::]
+
+        self.setup['search_plugin_found'] = False
+        encryptcontent_plugin_found = False
+        for plugin in config['plugins']:
+            if plugin.endswith('search'):
+                if encryptcontent_plugin_found:
+                    logger.error('Plugins need to be reordered ("search" ahead of "encryptcontent" in the end)! Otherwise search index might leak sensitive data.')
+                    os._exit(1) # prevent build without search index modification
+                self.setup['search_plugin_found'] = True
+            if plugin == 'encryptcontent':
+                encryptcontent_plugin_found = True
+        if not self.setup['search_plugin_found']:
+            logger.warning('"search" plugin wasn\'t enabled. Search index isn\'t generated or modified.')
+
+        self.setup['locations'] = {}
 
     def on_pre_build(self, config, **kwargs):
         """
@@ -281,6 +353,34 @@ class encryptContentPlugin(BasePlugin):
         except Exception as exp:
             logger.exception(exp)
 
+    def on_files(self, files, config, **kwargs):
+        """
+        The files event is called after the files collection is populated from the docs_dir.
+        Use this event to add, remove, or alter files in the collection.
+        Note that Page objects have not yet been associated with the file objects in the collection.
+        Use Page Events to manipulate page specific data.
+        """
+        if 'extensions' in self.config['hash_filenames']:
+            for file in files:
+
+                if 'except' in self.config['hash_filenames']:
+                    skip = False
+                    for check in self.config['hash_filenames']['except']:
+                        if file.src_path.endswith(check):
+                            skip = True
+                    if skip:
+                        continue
+
+                ext = file.src_path.rsplit('.',1)[1].lower()
+                if ext in self.config['hash_filenames']['extensions']:
+                    hash = self.__hash_md5_file__(file.abs_src_path)
+                    filename, ext =  file.abs_dest_path.rsplit('.',1)
+                    filename = filename + "_" + hash
+                    file.abs_dest_path = filename + "." + ext
+                    filename, ext =  file.url.rsplit('.',1)
+                    filename = filename + "_" + hash
+                    file.url = filename + "." + ext
+
     def on_page_markdown(self, markdown, page, config, **kwargs):
         """
         The page_markdown event is called after the page's markdown is loaded from file and
@@ -295,21 +395,23 @@ class encryptContentPlugin(BasePlugin):
         :return: Markdown source text of page as string
         """
 
+        encryptcontent = {} #init page data dict
+
         # Set global password as default password for each page
-        self.config['password'] = self.config['global_password']
+        encryptcontent['password'] = self.config['global_password']
 
         if 'password' in page.meta.keys():
             # If global_password is set, but you don't want to encrypt content
             page_password = str(page.meta.get('password'))
-            self.config['password'] = None if page_password == '' else page_password
+            encryptcontent['password'] = None if page_password == '' else page_password
             # Delete meta password information before rendering to avoid leak :]
             del page.meta['password']
 
         if 'use_secret' in page.meta.keys():
             if os.environ.get(str(page.meta.get('use_secret'))):
-                self.config['password'] = os.environ.get(str(page.meta.get('use_secret')))
+                encryptcontent['password'] = os.environ.get(str(page.meta.get('use_secret')))
             else:
-                if self.config['ignore_missing_secret'] and self.config['password']:
+                if self.config['ignore_missing_secret'] and encryptcontent['password']:
                     logger.warning('Cannot get password for "{page}" from environment variable: {var}. Using password from config or meta key as fallback!'.format(
                         var=str(page.meta.get('use_secret')), page=page.title)
                     )
@@ -319,6 +421,24 @@ class encryptContentPlugin(BasePlugin):
                     )
                     os._exit(1)                                 # prevent build without password to avoid leak0
 
+        if 'obfuscate' in page.meta.keys():
+            if encryptcontent['password'] is None: # Only allow obfuscation if no password defined
+                encryptcontent['obfuscate'] = True
+                encryptcontent['password'] = str(page.meta.get('obfuscate'))
+            del page.meta['obfuscate']
+
+        if encryptcontent.get('password'):
+            # Custom per-page strings
+            if 'encryption_summary' in page.meta.keys():
+                encryptcontent['summary'] = str(page.meta.get('encryption_summary'))
+                del page.meta['encryption_summary']
+
+            if 'encryption_info_message' in page.meta.keys():
+                encryptcontent['encryption_info_message'] = str(page.meta.get('encryption_info_message'))
+                del page.meta['encryption_info_message']
+
+            setattr(page, 'encryptcontent', encryptcontent)
+
         return markdown
 
     def on_page_content(self, html, page, config, **kwargs):
@@ -334,17 +454,16 @@ class encryptContentPlugin(BasePlugin):
         :param site_navigation: global navigation object
         :return: HTML rendered from Markdown source as string encrypt with AES
         """
-        if self.config['password'] is not None:
-            if self.config['title_prefix']:
-                page.title = str(self.config['title_prefix']) + str(page.title)
+
+        if hasattr(page, 'encryptcontent'):
             if self.config['tag_encrypted_page']:
                 # Set attribute on page to identify encrypted page on template rendering
                 setattr(page, 'encrypted', True)
-            # Set password attributes on page for other mkdocs events
-            setattr(page, 'password', str(self.config['password']))
+
             # Keep encrypted html to encrypt as temporary variable on page
             if not self.config['inject']:
-                setattr(page, 'html_to_encrypt', html)
+                page.encryptcontent['html_to_encrypt'] = html
+
         return html
 
     def on_page_context(self, context, page, config, **kwargs):
@@ -360,38 +479,60 @@ class encryptContentPlugin(BasePlugin):
         :param nav: global navigation object
         :return: dict of template context variables
         """
-        if hasattr(page, 'html_to_encrypt') or self.config['inject']:
+        if hasattr(page, 'encryptcontent'):
             if 'i18n_page_file_locale' in context:
                 locale = context['i18n_page_file_locale']
                 if locale in self.config['translations']:
                     translations = self.config['translations'][locale]
-                    if 'title_prefix' in translations:
-                        self.config['title_prefix'] = translations['title_prefix']
-                    if 'summary' in translations:
-                        self.config['summary'] = translations['summary']
-                    if 'placeholder' in translations:
-                        self.config['placeholder'] = translations['placeholder']
-                    if 'password_button_text' in translations:
-                        self.config['password_button_text'] = translations['password_button_text']
-                    if 'decryption_failure_message' in translations:
-                        self.config['decryption_failure_message'] = translations['decryption_failure_message']
-                    if 'encryption_info_message' in translations:
-                        self.config['encryption_info_message'] = translations['encryption_info_message']
-        if hasattr(page, 'html_to_encrypt'):
-            page.content = self.__encrypt_content__(
-                page.html_to_encrypt, 
-                str(page.password),
-                context['base_url']+'/',
-                self.config['site_path']+page.url
-            )
-            delattr(page, 'html_to_encrypt')
-        if self.config['inject']:
-            setattr(page, 'decrypt_form', self.__encrypt_content__(
-                '<!-- dummy -->', 
-                str(page.password),
-                context['base_url']+'/',
-                self.config['site_path']+page.url
-            ))
+
+                    #apply translation if defined
+                    if 'title_prefix' in translations and 'title_prefix' not in page.encryptcontent:
+                        page.encryptcontent['title_prefix'] = translations['title_prefix']
+                    if 'summary' in translations and 'summary' not in page.encryptcontent:
+                        page.encryptcontent['summary'] = translations['summary']
+                    if 'placeholder' in translations and 'placeholder' not in page.encryptcontent:
+                        page.encryptcontent['placeholder'] = translations['placeholder']
+                    if 'password_button_text' in translations and 'password_button_text' not in page.encryptcontent:
+                        page.encryptcontent['password_button_text'] = translations['password_button_text']
+                    if 'decryption_failure_message' in translations and 'decryption_failure_message' not in page.encryptcontent:
+                        page.encryptcontent['decryption_failure_message'] = translations['decryption_failure_message']
+                    if 'encryption_info_message' in translations and 'encryption_info_message' not in page.encryptcontent:
+                        page.encryptcontent['encryption_info_message'] = translations['encryption_info_message']
+
+            #init default strings from config
+            if 'title_prefix' not in page.encryptcontent:
+                page.encryptcontent['title_prefix'] = self.config['title_prefix']
+            if 'summary' not in page.encryptcontent:
+                page.encryptcontent['summary'] = self.config['summary']
+            if 'placeholder' not in page.encryptcontent:
+                page.encryptcontent['placeholder'] = self.config['placeholder']
+            if 'password_button_text' not in page.encryptcontent:
+                page.encryptcontent['password_button_text'] = self.config['password_button_text']
+            if 'decryption_failure_message' not in page.encryptcontent:
+                page.encryptcontent['decryption_failure_message'] = self.config['decryption_failure_message']
+            if 'encryption_info_message' not in page.encryptcontent:
+                page.encryptcontent['encryption_info_message'] = self.config['encryption_info_message']
+
+            if page.encryptcontent['title_prefix']: 
+                page.title = str(self.config['title_prefix']) + str(page.title)
+
+            if 'html_to_encrypt' in page.encryptcontent:
+                page.content = self.__encrypt_content__(
+                    page.encryptcontent['html_to_encrypt'], 
+                    context['base_url']+'/',
+                    self.setup['site_path']+page.url,
+                    page.encryptcontent
+                )
+                page.encryptcontent['html_to_encrypt'] = None
+
+            if self.config['inject']:
+                page.encryptcontent['decrypt_form'] = self.__encrypt_content__(
+                    '<!-- dummy -->', 
+                    context['base_url']+'/',
+                    self.setup['site_path']+page.url,
+                    page.encryptcontent
+                )
+
         return context
 
     def on_post_page(self, output_content, page, config, **kwargs):
@@ -407,10 +548,10 @@ class encryptContentPlugin(BasePlugin):
         :return: output of rendered template as string
         """
         # Limit this process only if encrypted_something feature is enable *(speedup)*
-        if (self.config['encrypted_something'] and hasattr(page, 'encrypted')
-                and len(self.config['encrypted_something']) > 0):  # noqa: W503
+        if (self.setup['encrypted_something'] and hasattr(page, 'encryptcontent')
+                and len(self.setup['encrypted_something']) > 0):  # noqa: W503
             soup = BeautifulSoup(output_content, 'html.parser')
-            for name, tag in self.config['encrypted_something'].items():
+            for name, tag in self.setup['encrypted_something'].items():
                 # logger.debug({'name': name, 'html tag': tag[0], 'type': tag[1]})
                 something_search = soup.findAll(tag[0], {tag[1]: name})
                 if something_search is not None and len(something_search) > 0:
@@ -428,11 +569,10 @@ class encryptContentPlugin(BasePlugin):
                         else:
                             merge_item = ""
                         # Encrypt child items on target tags with page password
-                        cipher_bundle = self.__encrypt_text_aes__(merge_item, str(page.password))
+                        cipher_bundle = self.__encrypt_text_aes__(merge_item, str(page.encryptcontent['password']))
                         encrypted_content = b';'.join(cipher_bundle).decode('ascii')
                         # Replace initial content with encrypted one
-                        bs4_encrypted_content = BeautifulSoup(encrypted_content, 'html.parser')
-                        item.contents = [bs4_encrypted_content]
+                        item.string = encrypted_content
                         if item.has_attr('style'):
                             if isinstance(item['style'], list):
                                 item['style'].append("display:none")
@@ -441,53 +581,22 @@ class encryptContentPlugin(BasePlugin):
                                 item['style'] = item['style'] + "display:none"
                         else:
                             item['style'] = "display:none"
+
             if self.config['inject'] and len(self.config['inject']) == 1:
                 name, tag = list(self.config['inject'].items())[0]
                 injector = soup.new_tag("div")
                 something_search = soup.find(tag[0], {tag[1]: name})
                 something_search.insert_before(injector)
-                injector.append(BeautifulSoup(page.decrypt_form, 'html.parser'))
-                delattr(page, 'decrypt_form')
+                injector.append(BeautifulSoup(page.encryptcontent['decrypt_form'], 'html.parser'))
+                page.encryptcontent['decrypt_form'] = None
+
             output_content = str(soup)
 
-        #TODO: Move search index processing to on_post_build to be less likely to fail due to incompatible search plugins
-        if hasattr(page, 'encrypted'):
-            #encrypt or exclude encrypted pages from search_index.json
-            for plugin in config['plugins']:
-                if plugin.endswith('search'):
-                    try:
-                        if hasattr(config['plugins'][plugin].search_index, '_entries'):
-                            search_entries = config['plugins'][plugin].search_index._entries
-                        elif hasattr(config['plugins'][plugin].search_index, 'entries'):
-                            search_entries = config['plugins'][plugin].search_index.entries
-                        else:
-                            logger.error('The search plugin "' + plugin + '" is currently unknown!')
+        if hasattr(page, 'encryptcontent'):
+            location = page.url.lstrip('/')
+            self.setup['locations'][location] = [page.encryptcontent['password'], page.encryptcontent.get('obfuscate')]
+            delattr(page, 'encryptcontent')
 
-                        for entry in search_entries.copy(): #iterate through all entries of search_index
-                            location = page.url.lstrip('/')
-                            if entry['location'] == location or entry['location'].startswith(location+"#"): #find the ones located at encrypted pages
-                                if self.config['search_index'] == 'encrypted':
-                                    search_entries.remove(entry)
-                                elif self.config['search_index'] == 'dynamically' and page.password is not None:
-                                    #encrypt text/title/location(anchor only)
-                                    text = entry['text']
-                                    title = entry['title']
-                                    toc_anchor = entry['location'].replace(location, '')
-                                    code = self.__encrypt_text_aes__(text, page.password)
-                                    entry['text'] = b';'.join(code).decode('ascii')
-                                    code = self.__encrypt_text_aes__(title, page.password)
-                                    entry['title'] = b';'.join(code).decode('ascii')
-                                    code = self.__encrypt_text_aes__(toc_anchor, page.password)
-                                    entry['location'] = location + ';' + b';'.join(code).decode('ascii')
-
-                        if hasattr(config['plugins'][plugin].search_index, '_entries'):
-                            config['plugins'][plugin].search_index._entries = search_entries
-                        elif hasattr(config['plugins'][plugin].search_index, 'entries'):
-                            config['plugins'][plugin].search_index.entries = search_entries
-
-                    except:
-                        logger.error('Could not encrypt search index of "' + page.title + '" for "' + plugin+ '" plugin! Abort !')
-                        os._exit(1)                                 # prevent build with possible plaintext in search_index
         return output_content
 
     def on_post_build(self, config, **kwargs):
@@ -502,13 +611,64 @@ class encryptContentPlugin(BasePlugin):
         with open(decrypt_js_path, "w") as file:
             file.write(self.__generate_decrypt_js__())
 
+        #modify search_index in the style of mkdocs-exclude-search
+        if self.setup['search_plugin_found'] and self.config['search_index'] != 'clear':
+            search_index_filename = Path(config.data["site_dir"] + "/search/search_index.json")
+            #TODO: Check if search_index.json exists and error if not found (moved its location)
+            with open(search_index_filename, "r") as f:
+                search_entries = json.load(f)
+
+            for entry in search_entries['docs'].copy(): #iterate through all entries of search_index
+                for location in self.setup['locations'].keys():
+                    if entry['location'] == location or entry['location'].startswith(location+"#"): #find the ones located at encrypted pages
+                        page_password = self.setup['locations'][location][0]
+                        if self.config['search_index'] == 'encrypted':
+                            search_entries.remove(entry)
+                        elif self.config['search_index'] == 'dynamically' and page_password is not None:
+                            #encrypt text/title/location(anchor only)
+                            text = entry['text']
+                            title = entry['title']
+                            toc_anchor = entry['location'].replace(location, '')
+                            code = self.__encrypt_text_aes__(text, page_password )
+                            entry['text'] = b';'.join(code).decode('ascii')
+                            code = self.__encrypt_text_aes__(title, page_password)
+                            entry['title'] = b';'.join(code).decode('ascii')
+                            code = self.__encrypt_text_aes__(toc_anchor, page_password)
+                            entry['location'] = location + ';' + b';'.join(code).decode('ascii')
+                        break
+
+            with open(search_index_filename, "w") as f:
+                json.dump(search_entries, f)
+            logger.info('Modified search_index.')
+
         # optionally download cryptojs
-        if self.config["selfhost"] and self.config["selfhost_download"]:
-            logger.info('Downloading cryptojs for self-hosting... Please consider copying "assets/javascripts/cryptojs/" to "doc/" and setting "selfhost_download: false" to decrease build time.')
-            Path(config.data["site_dir"] + '/assets/javascripts/cryptojs/').mkdir(parents=True, exist_ok=True)
+        if self.config['selfhost'] and self.config['selfhost_download']:
+            logger.info('Downloading cryptojs for self-hosting (if needed)...')
+            if self.config['selfhost_dir']:
+                configpath = Path(config['config_file_path']).parents[0]
+                dlpath = configpath.joinpath(self.config['selfhost_dir'] + '/assets/javascripts/cryptojs/')
+            else:
+                dlpath = Path(config.data['docs_dir'] + '/assets/javascripts/cryptojs/')
+            dlpath.mkdir(parents=True, exist_ok=True)
             for jsurl in JS_LIBRARIES:
-                dlurl = "https:" + jsurl
-                filepath = Path(config.data["site_dir"] + '/assets/javascripts/cryptojs/' + jsurl.rsplit('/',1)[1])
-                with urlopen(dlurl) as response:
-                    with open(filepath, 'wb') as file:
-                        file.write(response.read())
\ No newline at end of file
+                dlurl = "https:" + jsurl[0]
+                filepath = dlpath.joinpath(jsurl[0].rsplit('/',1)[1])
+                self.__download_and_check__(filepath, dlurl, jsurl[1])
+
+        passwords = set() #get all unique passwords
+        for location in self.setup['locations'].keys():
+            if not self.setup['locations'][location][1]:
+                passwords.add(self.setup['locations'][location][0])
+        self.setup['locations'].clear()
+        min_enttropy_spied_on, min_enttropy_secret = 0, 0
+        for password in passwords:
+            enttropy_spied_on, enttropy_secret = self.__get_entropy_from_password__(password)
+            if min_enttropy_spied_on == 0 or enttropy_spied_on < min_enttropy_spied_on:
+                min_enttropy_spied_on = enttropy_spied_on
+            if min_enttropy_secret == 0 or enttropy_secret < min_enttropy_secret:
+                min_enttropy_secret = enttropy_secret
+        if min_enttropy_spied_on < 100 and min_enttropy_spied_on > 0:
+            logger.warning('mkdocs-encryptcontent-plugin will always be vulnerable to brute-force attacks!'
+                           ' Your weakest password only got {spied_on} bits of entropy, if someone watched you while typing'
+                           ' (and a maximum of {secret} bits total)!'.format(spied_on = math.ceil(enttropy_spied_on), secret = math.ceil(min_enttropy_secret))
+                    )
